/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19 janvier 2007
   derniere modification le 20/03/2008 */

#ifndef SCESCENE_H
#define SCESCENE_H

#include <SCE/interface/SCEState.h>
#include <SCE/interface/SCECamera.h>
#include <SCE/interface/SCENode.h>
#include <SCE/interface/SCEModel.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*
  Gestionnaire de scenes.
  
  Morceau principal du moteur, gestion des objets dans la scene:
  - octree/quadtree
  - BSP
  - contribution culling
  - cameras
    - mouvements avances
    - multiples
  - changement d'etats:
    - changement des arbres de partitionnement a utiliser --> BSP, quadtree, ...
    - effet global a la scene --> HDR, bloom, glow, ...
  - LOD --> textures, modeles, terrain, ...
  - tri/ordre d'affichage:
    - textures
    - shaders
    - modeles
    - materiaux
  - screenshot --> DevIL pour l'enregistrement
  - video --> Open*L... a voir sur dvp.net (OpenCV .. ?)
  - rendu multi-passes --> HDR, bloom, post-effect, ...
  - IA de traitement des donnees (cf ordre d'affichage)
  - {occlusion culling
  - scene hierarchique ...
  - ...
*/

/* informations ponctuelles sur une scene (son etat de rendu par ex.) */
typedef struct sce_ssceneinfos SCE_SSceneInfos;
struct sce_ssceneinfos
{
    SCE_SCamera *camera;
};

typedef struct sce_sscene SCE_SScene;
/* fonction de rendu de la scene */
typedef void (*SCE_FRenderSceneFunc)(SCE_SScene*, void*);
struct sce_sscene
{
    SCE_SSceneInfos infos;           /* informations sur l'etat de la scene */
    SCE_SStateManager *statemanager; /* environnement des etats de la scene */
    SCE_FRenderSceneFunc render;     /* cible de rendu */

    SCE_STexture *rendertarget;

/*
    SCE_FSceneSetUpModels func2;
    SCE_FSceneBeginRender begin;
    SCE_FSceneFinishRender finish;
    SCE_FSceneSortByMaterial sortmat;
    SCE_FSceneSortByShader sortshader;
    SCE_FSceneSort... sort...;
*/

    SCE_SNode *rootnode; /* noeud racine */

    SCE_SList *models;   /* liste des modeles de la scene */
    SCE_SList *lights;   /* liste des lumieres de la scene */
    SCE_SList *cameras;  /* liste des cameras actives de la scene */

    int frustum_culling; /* booleen definissant l'usage du frustum culling */
    int use_skybox;      /* defini si on utilise une skybox */

    int *states, num_states; /* etats a appliquer lors du rendu */
};

#define SCE_Scene_ForEachState(i, s)\
    for (i=0; i<s->num_states; i++)
#define SCE_Scene_PushState(i, s)\
    SCE_State_Push (s->statemanager, s->states[i])
#define SCE_Scene_PopState(s)\
    SCE_State_Pop (s->statemanager)

/* initialise le gestionnaire de scenes */
int SCE_Init_Scene (void);
/* quitte le gestionnaire de scenes */
void SCE_Quit_Scene (void);

/* cree une nouvelle scene */
SCE_SScene* SCE_Scene_Create (void);
/* supprime une scene */
void SCE_Scene_Delete (SCE_SScene*);

/* determine l'etat du frustum culling */
void SCE_Scene_EnableFrustumCulling (SCE_SScene*);
void SCE_Scene_DisableFrustumCulling (SCE_SScene*);

/* determine si la scene utilise une skybox */
void SCE_Scene_EnableSkybox (SCE_SScene*);
void SCE_Scene_DisableSkybox (SCE_SScene*);

/* defini un tableau d'etats qui seront appliques les uns apres
   les autres lors du rendu (rendu multi-passes) */
void SCE_Scene_SetStates (SCE_SScene*, int*, int);

/* genere les boundings boxes de tous les modeles de la scene */
void SCE_Scene_GenerateBoundingsBoxes (SCE_SScene*);

/* retourne le noeud racine */
SCE_SNode* SCE_Scene_GetRootNode (SCE_SScene*);
/* retourne la camera specifiee */
SCE_SCamera* SCE_Scene_GetCamera (SCE_SScene*, unsigned int);

/* ajoute un noeud au noeud racine */
int SCE_Scene_AddNode (SCE_SScene*, SCE_SNode*);
/* ajoute un modele a une scene */
int SCE_Scene_AddModel (SCE_SScene*, SCE_SModel*);
/* ajoute une skybox a une scene */
int SCE_Scene_AddSkybox (SCE_SScene*, SCE_SModel*);
/* ajoute une lumiere a une scene */
int SCE_Scene_AddLight (SCE_SScene*, SCE_SLight*);
/* ajoute une camera a une scene */
int SCE_Scene_AddCamera (SCE_SScene*, SCE_SCamera*);

/* defini la fonction de rendu */
void SCE_Scene_SetRenderFunc (SCE_SScene*, SCE_FRenderSceneFunc);

/* defini et renvoie le "target" de rendu */
void SCE_Scene_SetRenderTarget (SCE_SScene*, SCE_STexture*);
SCE_STexture* SCE_Scene_GetRenderTarget (SCE_SScene*);


/* --- liste de fonctions outils pour manipuler le rendu --- */

/* fonction d'activation de toutes les lumieres
   se trouvant a portee du modele donne */
void SCE_Scene_ActiveLightsForModel (SCE_SScene*, SCE_SModel*);
/*void SCE_Scene_FrustumCulling.... ?*/
/* prepare la scene pour le rendu */
void SCE_Scene_PrepareRendering (SCE_SScene*);
/* calcul si un objet est dans le frustum de vue */
int SCE_Scene_ModelInFrustum (SCE_SScene*, SCE_SModel*);


/* callback de rendu par defaut d'un modele */
void SCE_Scene_RenderModel (SCE_SModel*, void*);
/* callback de rendu par defaut d'une skybox */
void SCE_Scene_RenderSkybox (SCE_SModel*, void*);

/* rend une scene */
void SCE_Scene_Render (SCE_SScene*, void*);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
